<?php

namespace Algolia\AlgoliaSearch\Model;

use Magento\Store\Model\StoreManagerInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewriteFactory;

class LandingPageUrlRewriteGenerator
{
    /**
     * Entity type code
     */
    const ENTITY_TYPE = 'landing-page';

    /**
     * @var \Magento\UrlRewrite\Service\V1\Data\UrlRewriteFactory
     */
    protected $urlRewriteFactory;

    /**
     * Store manager
     *
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Algolia\AlgoliaSearch\Model\LandingPage
     */
    protected $landingPage;

    /**
     * @param \Magento\UrlRewrite\Service\V1\Data\UrlRewriteFactory $urlRewriteFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        UrlRewriteFactory $urlRewriteFactory,
        StoreManagerInterface $storeManager
    ) {
        $this->urlRewriteFactory = $urlRewriteFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param \Algolia\AlgoliaSearch\Model\LandingPage $landingPage
     *
     * @return \Magento\UrlRewrite\Service\V1\Data\UrlRewrite[]
     */
    public function generate($landingPage)
    {
        $storeId = $landingPage->getStoreId();
        $this->landingPage = $landingPage;
        $urls = $storeId == 0 ? $this->generateForAllStores()
            : $this->generateForSpecificStore($storeId);
        $this->landingPage = null;

        return $urls;
    }

    /**
     * Generate list of urls for default store
     *
     * @return \Magento\UrlRewrite\Service\V1\Data\UrlRewrite[]
     */
    protected function generateForAllStores()
    {
        $urls = [];
        foreach ($this->storeManager->getStores() as $store) {
            $urls[] = $this->createUrlRewrite($store->getStoreId());
        }

        return $urls;
    }

    /**
     * Generate url for one store
     *
     * @param int $storeId
     *
     * @return \Magento\UrlRewrite\Service\V1\Data\UrlRewrite[]
     */
    protected function generateForSpecificStore($storeId)
    {
        $urls = [];
        $existingStores = $this->storeManager->getStores();
        if (!isset($existingStores[$storeId])) {
            return $urls;
        }
        $urls[] = $this->createUrlRewrite($storeId);

        return $urls;
    }

    /**
     * Create url rewrite object
     *
     * @param int $storeId
     * @param int $redirectType
     *
     * @return \Magento\UrlRewrite\Service\V1\Data\UrlRewrite
     */
    protected function createUrlRewrite($storeId, $redirectType = 0)
    {
        return $this->urlRewriteFactory->create()->setStoreId($storeId)
            ->setEntityType(self::ENTITY_TYPE)
            ->setEntityId($this->landingPage->getId())
            ->setRequestPath($this->landingPage->getUrlKey())
            ->setTargetPath('algolia/landingpage/view/landing_page_id/' . $this->landingPage->getId())
            ->setIsAutogenerated(1)
            ->setRedirectType($redirectType);
    }
}
